/*****************************************************************************
**                                    / _|
**   __ _ _   _ _ __ ___  _ __ __ _  | |_ ___  ___ ___
**  / _` | | | | '__/ _ \| '__/ _` | |  _/ _ \/ __/ __|
** | (_| | |_| | | | (_) | | | (_| | | || (_) \__ \__ \
**  \__,_|\__,_|_|  \___/|_|  \__,_| |_| \___/|___/___/
**
** Copyright (C) 2018 Aurora Free Open Source Software.
**
** This file is part of the Aurora Free Open Source Software. This
** organization promote free and open source software that you can
** redistribute and/or modify under the terms of the GNU Lesser General
** Public License Version 3 as published by the Free Software Foundation or
** (at your option) any later version approved by the Aurora Free Open Source
** Software Organization. The license is available in the package root path
** as 'LICENSE' file. Please review the following information to ensure the
** GNU Lesser General Public License version 3 requirements will be met:
** https://www.gnu.org/licenses/lgpl.html .
**
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3 or later as published by the Free Software
** Foundation. Please review the following information to ensure the GNU
** General Public License requirements will be met:
** http://www.gnu.org/licenses/gpl-3.0.html.
**
** NOTE: All products, services or anything associated to trademarks and
** service marks used or referenced on this file are the property of their
** respective companies/owners or its subsidiaries. Other names and brands
** may be claimed as the property of others.
**
** For more info about intellectual property visit: aurorafoss.org or
** directly send an email to: contact (at) aurorafoss.org .
*****************************************************************************/

#include <AuroraFW/IO/Timer.h>
#include <AuroraFW/CoreLib/Type.h>

namespace AuroraFW {
	namespace IO {
		Timer::Timer()
		{
			#ifdef AFW_TARGET_PLATFORM_WINDOWS
			LARGE_INTEGER freq;
			QueryPerformanceFrequency(&freq);
			_frequency = 1.0 / freq.QuadPart;
			#endif

			reset();
		}

		void Timer::reset()
		{
			#ifdef AFW_TARGET_PLATFORM_WINDOWS
			QueryPerformanceCounter(&_start);
			#elif defined(AFW_TARGET_ENVIRONMENT_POSIX)
			_start = HighResolutionClock::now();
			#endif
		}

		float Timer::elapsed()
		{
			#ifdef AFW_TARGET_PLATFORM_WINDOWS
			LARGE_INTEGER current;
			QueryPerformanceCounter(&current);
			uint64_t cycles = current.QuadPart - _start.QuadPart;
			return (float)(cycles * _frequency);
			#elif defined(AFW_TARGET_ENVIRONMENT_POSIX)
			return elapsedMillis() / 1000.0f;
			#endif
		}

		float Timer::elapsedMillis()
		{
			#ifdef AFW_TARGET_PLATFORM_WINDOWS
			return elapsed() * 1000.0f;
			#elif defined(AFW_TARGET_ENVIRONMENT_POSIX)
			return std::chrono::duration_cast<milliseconds_type>(HighResolutionClock::now() - _start).count();
			#endif
		}
	}
}
